import pydicom

import numpy as np


class Dcm2Numpy(object):

    def __init__(self):
        self._input_dicom_file_path = None
        self._output_numpy_array = None
        self._normalize_enabled = False
        self._shape = None

    def set_input_dicom_file_path(self, input_dicom_file_path):
        self._input_dicom_file_path = input_dicom_file_path

    def set_normalize_enabled(self, enabled=True):
        self._normalize_enabled = enabled

    def get_output_numpy_array(self):
        return self._output_numpy_array

    def get_shape(self):
        return self._shape

    def execute(self):
        if self._input_dicom_file_path is None:
            raise RuntimeError('Input DICOM file path not set')
        p = pydicom.read_file(self._input_dicom_file_path)
        self._shape = (p.Rows, p.Columns)
        self._output_numpy_array = p.pixel_array.reshape(self._shape)
        if self._normalize_enabled:
            b = p.RescaleIntercept
            m = p.RescaleSlope
            self._output_numpy_array = m * self._output_numpy_array + b
        # print('{}, {}'.format(np.min(self._output_numpy_array), np.max(self._output_numpy_array)))


if __name__ == '__main__':
    component = Dcm2Numpy()
    component.set_input_dicom_file_path('/Users/Ralph/data/Jorne/P0001.dcm')
    component.execute()
    print(component.get_output_numpy_array().shape)



import os
import cmd2
import numpy as np


class Result(object):

    def __init__(self, data, name):
        self.data = data
        self.name = name
        self.description = None


class ResultManager(object):

    def __init__(self):
        self.results = {}
        self.current_result = None
        self.result_idx = -1

    def next_result_idx(self):
        self.result_idx += 1
        return self.result_idx

    def add_result_data(self, data):
        name = 'result_{}'.format(self.next_result_idx())
        result = Result(data, name)
        self.results[name] = result
        self.current_result = self.results[name]

    def remove_result_data(self, name):
        self.current_result = self.results['result_0']
        del self.results[name]

    def get_current_result_data(self):
        return self.current_result.data

    def set_current_result(self, name):
        self.current_result = self.results[name]

    def set_result_description(self, name, description):
        self.results[name].description = description

    def undo(self):
        name = self.current_result.name
        index = int(name.split('_')[1])
        index = np.minimum(0, index-1)
        name = 'result_{}'.format(index)
        self.current_result = self.results[name]

    def redo(self):
        name = self.current_result.name
        index = int(name.split('_')[1])
        index = np.maximum(index+1, len(self.results.keys()))
        name = 'result_{}'.format(index)
        self.current_result = self.results[name]

    def show_results(self):
        current = ''
        for result in self.results.values():
            if result.name == self.current_result.name:
                current = '[current]'
            print('{}: description = {} {}'.format(result.name, result.description, current))


class BasicShell(cmd2.Cmd):

    def __init__(self):
        super(BasicShell, self).__init__()
        self.result_manager = ResultManager()
        self.current_dir = os.path.abspath(os.path.curdir)
        self.intro = 'Put your intro inside a global variable INTRO'
        self.prompt = '(shell) '

    def add_result(self, data):
        """
        Usage: add_result <data>
        Add <data> as a new result set to the result manager.
        """
        self.result_manager.add_result_data(data)

    def remove_result(self, name):
        """
        Usage: remove_result <result name>
        Remove result <name> from list of result sets.
        """
        self.result_manager.remove_result_data(name)

    def do_cd(self, line):
        """
        Usage: cd <dir path>
        Change the current directory to <dir path>.
        """
        if line == '.' or line == '':
            self.do_pwd(None)
            return
        if line == '..':
            self.current_dir = os.path.split(self.current_dir)[0]
            self.do_pwd(None)
            return
        if not os.path.isdir(line):
            line = os.path.join(self.current_dir, line)
            if not os.path.isdir(line):
                self.poutput('Directory {} is not a valid directory'.format(line))
                return
        self.current_dir = line
        self.do_pwd(None)

    def do_ls(self, _):
        """
        Usage: ls
        Lists the contents of the current directory. Same as shell command "!ls -lap".
        """
        self.do_shell('cd {}; ls -lap'.format(self.current_dir))

    def do_pwd(self, _):
        """
        Usage: pwd
        Show the current directory.
        """
        self.poutput(self.current_dir)

    def do_shell(self, line):
        """
        Usage: !<command>
        Execute shell command <command>. For example, !echo $HOME will display the user's HOME directory.
        """
        if line is None or line is '':
            self.poutput('Please specify a shell command preceded by! For example, !echo $HOME')
        else:
            self.poutput('Running shell command: {}'.format(line))
            output = os.popen(line).read()
            self.poutput(output)

    def do_show_results(self, _):
        """
        Usage: show_results
        Shows all result sets currently available.
        """
        self.result_manager.show_results()

    def do_set_result_desc(self, line):
        """
        Usage: set_result_desc <name=description>
        Set description of result set <name> to <description>.
        """
        items = [x.strip() for x in line.split('=')]
        name, description = items[0], items[1]
        self.result_manager.set_result_description(name, description)

    def do_undo(self, _):
        """
        Usage: undo
        Sets current result set to previous result set.
        """
        self.result_manager.undo()

    def do_redo(self, _):
        """
        Usage: redo
        Sets current result set to next result set.
        """
        self.result_manager.redo()

    @staticmethod
    def do_exit(_):
        """
        Usage: exit
        Quits the shell application (same as quit).
        """
        return True



"""Top-level package for Barbell2."""

__author__ = """Ralph Brecheisen"""
__email__ = 'ralph.brecheisen@gmail.com'
__version__ = '0.1.0'

from .castorclient import castorclient
from .createh5 import createh5
from .dicomconverttoraw import dicomconverttoraw
from .dicomexplorer import dicomexplorer



import os
import json
import pydicom
import argparse
from cmd2 import with_argument_list
from pydicom._dicom_dict import DicomDictionary
from barbell2.lib import BasicShell


INTRO = """
Welcome to the DICOM Explorer!
------------------------------
This tool allows you to explore lots of DICOM files at the same time. Type 'help' to view a short list of
commands. Type 'help <command>' to view details about a given command.
"""

PROMPT = '(shell) '


def is_dicom(file_path):
    if not os.path.isfile(file_path):
        return False
    try:
        with open(file_path, "rb") as f:
            return f.read(132).decode("ASCII")[-4:] == "DICM"
    except:
        return False


def get_institute_name(file_path):
    items = os.path.split(file_path)
    institute_name = os.path.split(items[0])[1]
    return institute_name


class DicomExplorerShell(BasicShell):

    def __init__(self):
        super(DicomExplorerShell, self).__init__()
        self.intro = INTRO
        self.prompt = PROMPT
        self.debug = True

    # LOADING FILES AND DIRECTORIES

    def do_load_file(self, file_path):
        """
        Usage: load_file <file name or path>
        Load a single DICOM file. If only the name is provided, it assumed that the file is located in the
        current directory.
        """
        if not os.path.isfile(file_path):
            file_path = os.path.join(self.current_dir, file_path)
        if not is_dicom(file_path):
            self.poutput('File is not DICOM')
            return
        self.poutput('Loading...')
        self.result_manager.add_result_data([file_path])
        self.poutput('Loading done')

    def do_load_dir(self, dir_path):
        """
        Usage: load_dir <dir name or path>
        Load (recursively) all DICOM files in the given directory. If only the directory name is given, it is
        assumed that the directory is located in the current directory.
        """
        if not os.path.isdir(dir_path):
            dir_path = os.path.join(self.current_dir, dir_path)
        self.poutput('Loading {}...'.format(dir_path))
        data = []
        for root, dirs, files in os.walk(dir_path):
            for f in files:
                if not f.startswith('._'):
                    f = os.path.join(root, f)
                    if is_dicom(f):
                        data.append(f)
                        self.poutput(f)
        self.result_manager.add_result_data(data)
        self.poutput('Loading done')

    def do_show_files(self, n):
        """
        Usage: show_files [n]
        Show all (or first n) files loaded in the current result set. If you want to show files from another result set
        select it first using the set_current_result command.
        """
        files = self.result_manager.get_current_result_data()
        n = -1 if n == '' else int(n)
        count = 0
        for f in files:
            self.poutput(f)
            if count == n:
                break
            count += 1
        self.poutput('Total nr. of files: {}'.format(len(files)))

    def do_lookup_tag(self, tag_name):
        """
        Usage: lookup_tag <tag name>
        Lookup tag <tag name> in the DICOM dictionary. You can specify only parts of a tag name, e.g.,
        "Transmit" will return multiple dictionary entries containing the word "Transmit" (like Transmit
        Coil Name). Being able to lookup tags comes in handy when you search for them in DICOM files.
        Note that (parts of) the tag name does not have to be case-sensitive.
        """
        for key, value in DicomDictionary.items():
            output = '{}: {}'.format(key, value)
            if tag_name == '':
                self.poutput(output)
            else:
                for item in value:
                    if tag_name in item:
                        self.poutput(output)

    def do_show_tag(self, tag_name):
        """
        Usage: show_tag <tag name>
        Show values for tag <tag name> in the currently loaded DICOM files.
        """
        files = self.result_manager.get_current_result_data()
        values = {}
        for f in files:
            p = pydicom.read_file(f)
            for tag in p.keys():
                if tag in list(DicomDictionary.keys()):
                    if tag_name == DicomDictionary[tag][4]:
                        v = p[tag].value
                        if v not in values.keys():
                            values[v] = 0
                        values[v] += 1
                        self.poutput('{}: {}'.format(tag_name, v))
                        break
        self.poutput('Unique values: {}'.format(values))

    def do_show_header(self, file_name):
        """
        Usage: show_header <file name>
        Show DICOM header of <file name>. If <file name> is left empty, this function will search for
        all file paths that contain the given file name. If only one is encountered, the DICOM header will
        be displayed. If multiple files are encountered, a list of those files is displayed so that you can
        select a specific one (with its full path).
        """
        files = self.result_manager.get_current_result_data()
        count = 0
        counted_files = []
        for f in files:
            if file_name in f:
                count += 1
                counted_files.append(f)
        if count == 0:
            if os.path.isfile(file_name):
                p = pydicom.read_file(file_name)
                self.poutput(p)
            else:
                self.poutput('Could not find file {}'.format(file_name))
        elif count == 1:
            p = pydicom.read_file(counted_files[0])
            self.poutput(p)
        elif count > 1:
            self.poutput('Found multiple files with same name:')
            for f in counted_files:
                self.poutput(f)
            self.poutput('Please select full file path of file you want and repeat this command')

    def do_check_pixels(self, verbose):
        """
        Usage: check_pixels [verbose]
        Check that each DICOM file contains pixel values that can be loaded using pydicom and NumPy. Sometimes,
        images may be compressed in some way (e.g., using JPEG2000). In that case, they cannot be loaded with
        pydicom and their pixel values need to be extracted to raw format. You can use the dicomconverttoraw tool
        for that purpose.
        """
        files = self.result_manager.get_current_result_data()
        count = 0
        for f in files:
            p = pydicom.read_file(f)
            try:
                p.convert_pixel_data()
                if verbose != '':
                    self.poutput('OK: {}'.format(f))
            except NotImplementedError:
                count += 1
                self.poutput('ERROR: could not load pixel data {}'.format(f))
        if count > 0:
            self.poutput('Pixel data for {} out of {} files could not be loaded'.format(count, len(files)))
        else:
            self.poutput('Pixel data OK for all files')


def main():
    import sys
    shell = DicomExplorerShell()
    sys.exit(shell.cmdloop())


if __name__ == '__main__':
    main()



"""
Main module.
What should it do? Perhaps offer the user the possibility to see what's in the package?
Showing a list of available tools? Like createh5, castorclient, dicomconverttoraw, dicomexplorer, etc.
"""



from .dcm2masks import Dcm2Masks
from .dcm2nifti import Dcm2Nifti
from .dcm2numpy import Dcm2Numpy
from .nifti2masks import Nifti2Masks
from .tag2dcm import Tag2Dcm
from .tag2nifti import Tag2Nifti
from .tag2numpy import Tag2NumPy



import os
import sys
import argparse

from barbell2.utils import MyException


def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('source_dir', help='Source directory containing DICOM files to be converted')
    parser.add_argument('target_dir', help='Target directory where converted DICOM files will be copied to')
    args = parser.parse_args()
    return args


def run():

    # Get command-line arguments
    args = get_args()

    # Check that the gdcmconv tool exists
    result = os.system('which gdcmconv>/dev/null')
    if result > 0:
        raise MyException('Tool "gdcmconv" is probably not installed')

    # Check that source directory exists and is not empty. The target directory is not allowed to exist
    if not os.path.isdir(args.source_dir):
        raise MyException('Source directory "{}" does not exist'.format(args.source_dir))
    if len(os.listdir(args.source_dir)) == 0:
        raise MyException('Source directory "{}" is empty'.format(args.source_dir))
    if os.path.isdir(args.target_dir):
        raise MyException('Target directory "{}" exists. Please delete it first!'.format(args.target_dir))
    os.makedirs(args.target_dir, exist_ok=False)

    # Process DICOM files
    for file_name in os.listdir(args.source_dir):
        file_source_path = os.path.join(args.source_dir, file_name)
        file_target_path = os.path.join(args.target_dir, file_name)
        command = 'gdcmconv --raw {} {}'.format(file_source_path, file_target_path)
        os.system(command)
        print(command)


def main():
    try:
        run()
    except MyException as e:
        print(e)


if __name__ == '__main__':
    sys.argv = [
        'dicomconverttoraw.py',
        '/Volumes/USB_SECURE1/data/radiomics/projects/leanne/raw_anon_gdcm',
        '/Volumes/USB_SECURE1/data/radiomics/projects/leanne/raw_anon_gdcm_uncompressed',
    ]
    main()
